/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.eventmesh.runtime.admin.handler;

import org.apache.eventmesh.runtime.admin.controller.HttpHandlerManager;

import com.sun.net.httpserver.HttpHandler;

import lombok.Data;

/**
 * An abstract class that implements the {@link HttpHandler} interface
 * and provides basic functionality for HTTP request handling.
 * <p>
 * Subclasses should extend this class to implement specific HTTP request handling logic.
 */

@Data
public abstract class AbstractHttpHandler implements HttpHandler {

    private final HttpHandlerManager httpHandlerManager;

    public AbstractHttpHandler(HttpHandlerManager httpHandlerManager) {
        this.httpHandlerManager = httpHandlerManager;
        this.httpHandlerManager.register(this);
    }
}
